@@ -133,9 +133,7 @@ def sigmoid4_log(x, x0, r, amp, y0):
     y : nparray of float [N_x]
         Function value at argument x
     """
-    y = np.log10(y0 + (amp - y0) / (1 + np.exp(-r * (x - x0))))
-
-    return y
+    return np.log10(sigmoid4(y0=y0, amp=amp, x=x, x0=x0, r=r))
 
 
 def linear(x, m, n):
