@@ -229,10 +229,10 @@ def map_data_to_surface(datasets, points_datasets, con_datasets, fname_fsl_gm, f
         GM/WM surface (has to be provided for each dataset)
     con_datasets : nparray of int [N_tri x 3] or list of nparray
         Connectivity matrix of dataset points (has to be provided for each dataset)
-    fname_fsl_gm : str or list of str
+    fname_fsl_gm : str or list of str or list of None
         Filename of pial surface fsl file(s) (one or two hemispheres)
         e.g. in mri2msh: .../fs_ID/surf/lh.pial
-    fname_fsl_wm : str or list of str
+    fname_fsl_wm : str or list of str or list of None
         Filename of wm surface fsl file(s) (one or two hemispheres)
         e.g. in mri2msh: .../fs_ID/surf/lh.white
     fname_midlayer : str or list of str
@@ -1294,11 +1294,11 @@ class TetrahedraLinear:
                                                  P3_all[tet_idx, :],
                                                  points_out[i, :][np.newaxis], )
 
-            Vsub = np.array([Vsub1, Vsub2, Vsub3, Vsub4])
-            Vtot = np.sum(Vsub)
+            vsub = np.array([Vsub1, Vsub2, Vsub3, Vsub4])
+            vtot = np.sum(vsub)
 
             # calculate phi_out
-            qoi_out[i, ] = 1.0 * np.dot(Vsub.T, qoi[self.tetrahedra[tet_idx[0], :],]) / Vtot
+            qoi_out[i, ] = 1.0 * np.dot(vsub.T, qoi[self.tetrahedra[tet_idx[0], :],]) / vtot
 
             stop = time.time()
             print(('Total: Point: {:d}/{:d} [{} sec]\n'.format(i + 1, N_phi_points_out, stop - start)))
