@@ -10,7 +10,6 @@ import trimesh
 import tqdm
 from pynibs.util import sample_sphere
 from itertools import product
-from simnibs import SIMNIBSDIR, mesh_io, transformations
 
 
 def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
@@ -1391,6 +1390,7 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
     ValueError
         If the final ROI is empty.
     """
+    from simnibs import SIMNIBSDIR
     if outside_radius is None:
         outside_radius = np.inf
     # load image for affine and tissue borders
@@ -1580,6 +1580,7 @@ def nii2msh(mesh, m2m_dir, nii, out_folder, hem, out_fsaverage=False, roi_name='
     roi_name : str
         How to name the ROI
     """
+    from simnibs import mesh_io, transformations
     if isinstance(mesh, str):
         mesh = mesh_io.read_msh(mesh)
     if isinstance(nii, str):
