@@ -1,10 +1,8 @@
 import os
 import re
 import warnings
-import nibabel
 import copy
 import numpy as np
-from simnibs import mesh_io, read_msh
 import trimesh
 
 from .util import calc_tri_surface, calc_tet_volume
@@ -356,6 +354,7 @@ def read_coil_geo(fn_coil_geo):
 
 
 def check_mesh(mesh, verbose=False):
+    from simnibs import read_msh
     """
     Check a simmibs.Mesh for degenerated elements:
         - zero surface triangles
@@ -408,6 +407,7 @@ def check_mesh(mesh, verbose=False):
 
 
 def fix_mesh(mesh, verbose=False):
+    from simnibs import read_msh
     """
     Fixes simnibs.Mesh by removing any zero surface tris and zero volume tets and by fixing negative volume tets.
 
@@ -451,6 +451,7 @@ def fix_mesh(mesh, verbose=False):
 
 
 def smooth_mesh(mesh, output_fn, smooth=.8, approach='taubin', skin_only_output=False):
+    from simnibs import mesh_io
     """
     Smoothes the skin compartment of a simnibs mesh. Uses one of three trimesh.smoothing approaches.
 
