@@ -305,17 +305,17 @@ class Mesh:
         self.subject_folder = subject_folder
         self.name = mesh_name
         self.info = None
-        self.approach = None # 'mri2mesh', 'headreco'
+        self.approach = None  # 'mri2mesh', 'headreco', 'charm'
         self.mri_idx = None
 
         # default parameters
         self.mesh_folder = os.path.join(subject_folder, 'mesh', mesh_name)
         self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{subject_id}.msh")
         self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{subject_id}.hdf5")
-        self.fn_tensor_vn = f"d2c_{subject_id}/dti_results_T1space/DTI_conf_tensor.nii.gz"
+        self.fn_tensor_vn = f"d2c_{subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
         self.fn_mri_conform = f"{subject_id}_T1fs_conform.nii.gz"
-        self.fn_lh_midlayer = f"fs_{subject_id}/surf/lh.central"
-        self.fn_rh_midlayer = f"fs_{subject_id}/surf/rh.central"
+        self.fn_lh_midlayer = f"fs_{subject_id}{os.sep}surf{os.sep}lh.central"
+        self.fn_rh_midlayer = f"fs_{subject_id}{os.sep}surf{os.sep}rh.central"
         self.vertex_density = 1.0 # headreco
         self.numvertices = 100000 # mri2mesh
 
@@ -333,6 +333,11 @@ class Mesh:
         self.fn_lh_gm_curv = None
         self.fn_rh_gm_curv = None
 
+        # charm meshes
+        self.smooth_skin = None
+        self.refinement_roi = None
+        self.refinemement_element_size = None
+
     def fill_defaults(self, approach):
         """
         Initializes attributes for a headreco mesh.
@@ -341,17 +346,36 @@ class Mesh:
         -----------
         approach: str
             'headreco'
+            'mri2mesh'
+            'charm'
         """
         self.approach = approach
         if approach == 'headreco':
             self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
             self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
-            self.fn_tensor_vn = f"d2c_{self.subject_id}/dti_results_T1space/DTI_conf_tensor.nii.gz"
+            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
             self.fn_mri_conform = f"{self.subject_id}_T1fs_conform.nii.gz"
-            self.fn_lh_midlayer = f"fs_{self.subject_id}/surf/lh.central"
-            self.fn_rh_midlayer = f"fs_{self.subject_id}/surf/rh.central"
-        # else:
-        #     # raise NotImplementedError(f"Approach {approach} not implemented.")
+            self.fn_lh_midlayer = f"fs_{self.subject_id}{os.sep}surf{os.sep}lh.central"
+            self.fn_rh_midlayer = f"fs_{self.subject_id}{os.sep}surf{os.sep}rh.central"
+
+        elif approach == 'mri2mesh':
+            self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
+            self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
+            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
+            self.fn_mri_conform = f"{self.subject_id}_T1fs_conform.nii.gz"
+            self.fn_lh_gm_curv = f"fs_{self.subject_id}{os.sep}surf{os.sep}lh.curv.pial"
+            self.fn_rh_gm_curv = f"fs_{self.subject_id}{os.sep}surf{os.sep}rh.curv.pial"
+
+        elif approach == 'charm':
+            self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
+            self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
+            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
+            self.fn_mri_conform = f"{self.subject_id}_T1.nii.gz"
+            self.fn_lh_midlayer = f"m2m_{self.subject_id}{os.sep}surfaces{os.sep}lh.central.gii"
+            self.fn_rh_midlayer = f"m2m_{self.subject_id}{os.sep}surfaces{os.sep}rh.central.gii"
+
+        else:
+            raise NotImplementedError(f"Approach {approach} not implemented.")
 
     def write_to_hdf5(self, fn_hdf5, check_file_exist=False, verbose=False):
         """
@@ -563,7 +587,7 @@ def save_subject_hdf5(subject_id, subject_folder, fname, mri_dict=None, mesh_dic
         Hide warnings.
     verbose : bool
         Print information about meshes and ROIs.
-    kwargs : str or np.array
+    kwargs : str or np.ndarray
         Additional information saved in the parent folder of the .hdf5 file
 
     Returns
@@ -586,7 +610,7 @@ def save_subject_hdf5(subject_id, subject_folder, fname, mri_dict=None, mesh_dic
             mri_dict = {i: mri_dict[i] for i in range(len(mri_dict))}
 
         for i in mri_dict.keys():
-            write_dict_to_hdf5(fn_hdf5=fname, data=mri_dict[i], folder=f"mri/{i}", check_file_exist=check_file_exist)
+            write_dict_to_hdf5(fn_hdf5=fname, data=mri_dict[i], folder=f"mri/{i}", check_file_exist=True)
 
     if mesh_dict is not None:
         if isinstance(mesh_dict, list):
