@@ -15,6 +15,74 @@ from scipy.interpolate import griddata
 from sklearn.neighbors import KernelDensity
 
 
+def calc_tet_volume(points, abs=True):
+    """
+    Calculate tetrahedra volumes.
+
+    Parameter
+    ---------
+    points: np.ndarray
+        shape: (n_tets,4,3)
+        [[[Ax, Ay, Az],
+          [Bx, By, Bz],
+          [Cx, Cy, Cz],
+          [Dx, Dy, Dy]],
+
+         [[Ax, Ay, Az],
+          [Bx, By, Bz],
+          [Cx, Cy, Cz],
+          [Dx, Dy, Dy]],
+
+        ...
+
+        ]
+
+    Returns
+    -------
+    volume: np.ndarray
+        shape: (n_tets)
+
+    Other Parameters
+    ----------------
+    abs : bool
+        Return magnitude. Default: True.
+    """
+    if points.ndim == 2:
+        points = np.atleast_3d(points).reshape(1,4,3)
+    if points.ndim != 3:
+        raise ValueError(f"Wrong dimensions for points: ({points.shape}). Expected: (n_tets,4,3))")
+
+    a = np.zeros((points.shape[0], 4, 4))
+    a[:, :, 3] = 1
+    a[:, :, :3] = points
+    a = a.swapaxes(1, 2)
+
+    if abs:
+        return np.abs(1 / 6 * np.linalg.det(a))
+    else:
+        return 1 / 6 * np.linalg.det(a)
+
+
+def calc_tri_surface(points):
+    """
+    Calculate triangle surface areas.
+
+    Parameter
+    ---------
+    points : np.ndarray
+        (n_triangles,3,3)
+
+    Returns
+    -------
+    triange_area : np.ndarray
+    """
+    a = np.linalg.norm(points[:, 0] - points[:, 1], axis=1)
+    b = np.linalg.norm(points[:, 1] - points[:, 2], axis=1)
+    c = np.linalg.norm(points[:, 0] - points[:, 2], axis=1)
+    s = np.sum((a, b, c), axis=0) / 2
+    return (s * (s - a) * (s - b) * (s - c)) ** 0.5
+
+
 def rd(array, array_ref):
     """
     Determine the relative difference between input data and reference data.
@@ -33,7 +101,7 @@ def rd(array, array_ref):
         Relative difference between the columns of array and array_ref
     """
 
-    return np.linalg.norm(array-array_ref)/np.linalg.norm(array_ref)
+    return np.linalg.norm(array - array_ref) / np.linalg.norm(array_ref)
 
 
 def get_sphere(mesh=None, mesh_fn=None, target=None, radius=None, roi_idx=None, roi=None, elmtype='tris'):
@@ -108,7 +176,6 @@ def tris_in_sphere(mesh, target, radius, roi):
         return tri_target_idx
 
 
-
 def sample_sphere(n_points, r):
     """
     Creates n_points evenly spread in a sphere of radius r.
@@ -554,7 +621,6 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
     neigh_style : str, default: 'point'
         Should neighbors share point or 'edge'
 
-
     Returns
     -------
     idx_disc : list of int [n_disc]
@@ -578,7 +644,7 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
         raise NotImplementedError(f"neigh_style {neigh_style} unknown.")
 
     if crit == 'median':
-        def is_neigh(elm_i):
+        def is_neigh():
             if not (1 / deviation_factor < data_i / median < deviation_factor):
                 neighbor_indices = np.where(mask_neighbor)[0]
                 best_neigh = neighbor_indices[(np.abs(data[neighbor_indices] - median)).argmin()]
@@ -587,13 +653,23 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
                 # if neighbor:
                 idx_neighbor.append(best_neigh)
     elif crit == 'max':
-        def is_neigh(elm_i):
+        def is_neigh():
             if data_i / median < 1 / deviation_factor:
                 neighbor_indices = np.where(mask_neighbor)[0]
                 best_neigh = neighbor_indices[(data[neighbor_indices]).argmax()]
                 if min_val is None or data[best_neigh] > min_val:
                     idx_disc.append(elm_i)
 
+                # if neighbor:
+                idx_neighbor.append(best_neigh)
+    elif crit == 'randmax':
+        def is_neigh():
+            if data_i / median < 1 / deviation_factor:
+                neighbor_indices = np.where(mask_neighbor)[0]
+                best_neigh = np.random.choice(neighbor_indices[(data[neighbor_indices]) > 0], 1)
+                if min_val is None or data[best_neigh] > min_val:
+                    idx_disc.append(elm_i)
+
                 # if neighbor:
                 idx_neighbor.append(best_neigh)
     else:
@@ -601,42 +677,44 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
 
     # start = time.time()
     for elm_i, data_i in zip(range(n_ele), data):
-        if elm_i not in not_fitted_elms:
-            # find neighbors
-            mask = np.sum(np.isin(con, con[elm_i, :]), axis=1)
-            # mask_neighbor = np.logical_and(0 < mask, mask < 3)
-            mask_neighbor = get_neigh(mask)
-            # best_values are set to 0 for bad elements and unfittable ones. do not use these as neighbors
-            if not_fitted_elms is not None and not len(not_fitted_elms) == 0:
-                mask_neighbor[not_fitted_elms] = False
+        if elm_i in not_fitted_elms:
+            continue
 
-            # if the element is lonely floating and has no neighbors ... continue
-            if np.sum(mask_neighbor) == 0:
-                continue
+        # find neighbors
+        mask = np.sum(np.isin(con, con[elm_i, :]), axis=1)
+        mask_neighbor = get_neigh(mask)
 
-            # check if current value does not fit to neighbors
-            median = np.median(data[mask_neighbor])
+        # best_values are set to 0 for bad elements and unfittable ones. do not use these as neighbors
+        if not_fitted_elms is not None and len(not_fitted_elms) != 0:
+            mask_neighbor[not_fitted_elms] = False
 
-            if median == 0:
-                median = 1e-12
+        # if the element is lonely floating and has no neighbors ... continue
+        if not np.sum(mask_neighbor):
+            continue
 
-            is_neigh(elm_i)
+        # check if current value does not fit to neighbors
+        median = np.median(data[mask_neighbor])
 
-            # if not (1 / deviation_factor < data_i / median < deviation_factor):
-            #     # find best neighbor idx
-            #     neighbor_indices = np.where(mask_neighbor)[0]
-            #
-            #     if crit == 'max':
-            #         best_neigh = neighbor_indices[(data[neighbor_indices]).argmax()]
-            #     elif crit == 'median':
-            #         best_neigh = neighbor_indices[(np.abs(data[neighbor_indices] - median)).argmin()]
-            #     else:
-            #         raise  NotImplementedError(f"Criterium {crit} unknown. ")
-            #     if min_val is None or data[best_neigh] > min_val:
-            #         idx_disc.append(elm_i)
-            #
-            #     if neighbor:
-            #         idx_neighbor.append(best_neigh)
+        if not median:
+            median = 1e-12
+
+        is_neigh()
+
+        # if not (1 / deviation_factor < data_i / median < deviation_factor):
+        #     # find best neighbor idx
+        #     neighbor_indices = np.where(mask_neighbor)[0]
+        #
+        #     if crit == 'max':
+        #         best_neigh = neighbor_indices[(data[neighbor_indices]).argmax()]
+        #     elif crit == 'median':
+        #         best_neigh = neighbor_indices[(np.abs(data[neighbor_indices] - median)).argmin()]
+        #     else:
+        #         raise  NotImplementedError(f"Criterium {crit} unknown. ")
+        #     if min_val is None or data[best_neigh] > min_val:
+        #         idx_disc.append(elm_i)
+        #
+        #     if neighbor:
+        #         idx_neighbor.append(best_neigh)
 
         # stop = time.time()
         # print(stop-start)
@@ -647,7 +725,6 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
         return idx_disc
 
 
-
 def find_nearest(array, value):
     """
     Given an "array", and given a "value" , returns an index j such that "value" is between array[j]
@@ -1111,6 +1188,7 @@ def calc_n_network_combs(n_e, n_c, n_i):
 
     return binom(n_e, n_i) * np.sum([((n_i - 1) ** k) * binom(n_c, k) for k in range(1, n_c)])
 
+
 def cell_data_to_point_data(tris, data_tris, nodes, method='nearest'):
     """
     A wrapper for scipy.interpolate.griddata to interpolate cell data to node data.
@@ -1136,7 +1214,7 @@ def cell_data_to_point_data(tris, data_tris, nodes, method='nearest'):
     return griddata(elms_center, data_tris, nodes, method)
 
 
-def load_muaps(fn_muaps,fs=1e6,fs_downsample=1e5):
+def load_muaps(fn_muaps, fs=1e6, fs_downsample=1e5):
     # load MUAPs and downsample
     with h5py.File(fn_muaps, "r") as f:
         muaps_orig = f["MUAPShapes"][:]
@@ -1149,4 +1227,4 @@ def load_muaps(fn_muaps,fs=1e6,fs_downsample=1e5):
     for i in range(N_MU):
         muaps[:, i] = np.interp(t_muap, t_muap_orig, muaps_orig[:, i])
 
-    return muaps,t_muap
\ No newline at end of file
+    return muaps, t_muap
